import 'package:unittest/unittest.dart';
import 'package:clean_sync/server.dart';
import 'package:clean_sync/transactor_server.dart';
import 'package:clean_lock/lock_requestor.dart';
import 'dart:async';
import 'package:clean_ajax/server.dart';
import 'package:clean_ajax/client.dart';
import 'package:clean_sync/client.dart';
import 'package:clean_data/clean_data.dart';
import 'package:clean_sync/id_generator.dart';
import 'package:logging/logging.dart';

Logger _logger = new Logger('mongo_wrapper_logger');

main() {
  Logger.root.level = Level.INFO;
  Logger.root.onRecord.listen(print);
  run();
}

run() {
  MongoConnection mongoConnection;
  TransactorServer transactorServer;
  TransactorClient transactorClient;
  DataReference updateLock;

  LockRequestor lockRequestor;
  Map january, february, march, april, may, june, july,
      august, september, october, november, december;
  List monthsCol;
  var mongoUrl = 'mongodb://127.0.0.1/mongoProviderTest';
  var host = "127.0.0.1";
  var msPort = 27001;
  var lockerPort = 27002;
  setUp(() {

    return LockRequestor.connect(host, lockerPort)
       .then((LockRequestor _lockRequestor) => lockRequestor = _lockRequestor)
       .then((_) => mongoConnection = new MongoConnection(mongoUrl, lockRequestor))
       .then((_) => mongoConnection.init())
       .then((_) => transactorServer = new TransactorServer(mongoConnection))
       .then((_) => transactorServer.init())
       .then((_) {
            updateLock = new DataReference(false);
            MultiRequestHandler requestHandler = new MultiRequestHandler();
            requestHandler.registerHandler('sync-operation', transactorServer.handleSyncRequest);
            Connection connection = new Connection.config(
                new LoopBackTransportStub(requestHandler.handleLoopBackRequest,null));
            transactorClient = new TransactorClient(connection, updateLock, "author", new IdGenerator());

    });
  });

  test('Test operations with various process time', (){
    var times = new Iterable.generate(20, (x) => x*50);
    return Future.forEach(times, (t) {
      transactorServer.registerOperation('time$t',before: (_) => true, operation: (_) => new Future.delayed(new Duration(milliseconds: t)));
      transactorClient.registerClientOperation('time$t');
      return transactorClient.performServerOperation('time$t', {});
    });
  });

  tearDown(() {
    return Future.wait([transactorServer.close(), mongoConnection.close(), lockRequestor.close()]);
  });

}